#ifndef SCENETREE_H
#define SCENETREE_H

#include <vector>
#include <string>
#include "scene.h"
#include "containernode.h"
#include "engine.h"

class Light;
class CameraNode;

/**
 * @brief The SceneNode class represents a class of the game
 * that will be updated or used for rendering every frame
 */

class SceneTree : public Scene
{
public:
    ~SceneTree();

    virtual SceneIterator<Light*>* getLights();
    virtual SceneIterator<GeometryNode*>* getGeometry();

    void update();

    Texture* getSkybox() {return m_skybox;}
    void setSkybox(Texture* skybox) {m_skybox = skybox;}

    void setMainCamera(CameraNode *camNode);

    ContainerNode* getRootObject(){return &m_root;}
    void addToIndex(SceneNode* node);
    void removeFromIndex(SceneNode *node);
    void updateShaders();

    const Engine& getEngine() { return m_engine; }

private:
    // A SceneTree can only be constructed by the engine by using Engine::createScene()
    friend SceneTree* Engine::createScene();
    SceneTree(const Engine &engine);

    const Engine &m_engine;
    ContainerNode m_root;
    std::vector<Light*> m_lights;
    std::vector<GeometryNode*> m_geometries;
    Texture* m_skybox;
};

#endif // SCENETREE_H
