#ifndef SPARROWSHELL_H
#define SPARROWSHELL_H

#include <list>

#include "scene/scenetree.h"
#include "glm/vec2.hpp"
#include "sparrowshell/shellbuffer.h"
#include "sparrowshell/shellscrollbar.h"
#include "scene/graphicalcontainernode.h"
#include "keybindings.h"

class Input;
class MeshNode;
class TextNode;
class ShellBuffer;

namespace sf {
class Window;
}

class SparrowShell : public GraphicalContainerNode
{
private:
    glm::ivec2 m_position;
    glm::ivec2 m_dimension;
    bool m_shellEnabled = false;
    bool m_resizeBuffer = false;
    bool m_indexMoved = false;

    MeshNode* m_background;
    ShellBuffer* m_buffer;
    ShellScrollBar* m_scrollbar;

    glm::vec3 m_text_color;
    std::string m_input_string;
    unsigned int m_input_cursor_pos;
    TextNode* m_input_mesh;

    int m_plop_test;
    int m_move_cursor_left;
    int m_move_cursor_right;
    int m_clear_console_action;
    std::string m_previous_context;

public:
    static const unsigned int BUFFER_MAX_LENGTH;
    static const unsigned int BUFFER_DISPLAYED_LINES;
    static const unsigned int SCROLLBAR_PIXEL_WIDTH;
    static const float SHELL_DEPTH;
    static const float DEFAULT_FONT_SIZE;

    SparrowShell(sf::Window*);

    void update();
    void updateTextInput();
    void scrollUp();
    void scrollDown();

    void toggleShell();
    void out(std::string str);
    void out(std::string str, glm::vec3 color);

    glm::ivec2 getPosition(){return m_position;}
    glm::ivec2 getDimension(){return m_dimension;}

    unsigned int getIndex(){return m_buffer->getIndex();}
    ShellBuffer* getBuffer(){return m_buffer;}

    void setMoveCursorLeftAction(int action){m_move_cursor_left = action;}
    void setMoveCursorRightAction(int action){m_move_cursor_right = action;}
    void setPlopTest(int action){m_plop_test = action;}
    void setClearConsoleAction(int action){m_clear_console_action = action;}

    void moveCursorLeft(){m_input_cursor_pos--;}
    void moveCursorRight(){m_input_cursor_pos++;}

    bool isEnabled(){return m_shellEnabled;}
    bool isBufferResized(){return m_resizeBuffer;}
    bool indexMoved(){return m_indexMoved;}
};
#endif // SPARROWSHELL_H
